@@ -704,14 +704,20 @@ enum {
    PROP_VSCROLL_POLICY,
 };
 
+enum {
+  RESIZE,
+  LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
+
 static void swt_fixed_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 static void swt_fixed_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void swt_fixed_finalize (GObject *object);
 static void swt_fixed_map (GtkWidget *widget);
-static AtkObject *swt_fixed_get_accessible (GtkWidget *widget);
 static void swt_fixed_measure (GtkWidget *widget, GtkOrientation  orientation, int for_size, int *minimum,
 		int *natural, int *minimum_baseline, int *natural_baseline);
-static void swt_fixed_size_allocate (GtkWidget *widget, const GtkAllocation *allocation, int baseline);
+static void swt_fixed_size_allocate (GtkWidget *widget, int width, int height, int baseline);
 
 G_DEFINE_TYPE_WITH_CODE (SwtFixed, swt_fixed, GTK_TYPE_WIDGET,
 		G_IMPLEMENT_INTERFACE (GTK_TYPE_SCROLLABLE, NULL)
@@ -726,7 +732,7 @@ static void swt_fixed_class_init (SwtFixedClass *class) {
 	gobject_class->get_property = swt_fixed_get_property;
 	gobject_class->finalize = swt_fixed_finalize;
 
-	/* Scrollable implemetation */
+	/* Scrollable implementation */
 	g_object_class_override_property (gobject_class, PROP_HADJUSTMENT,    "hadjustment");
 	g_object_class_override_property (gobject_class, PROP_VADJUSTMENT,    "vadjustment");
 	g_object_class_override_property (gobject_class, PROP_HSCROLL_POLICY, "hscroll-policy");
@@ -737,8 +743,12 @@ static void swt_fixed_class_init (SwtFixedClass *class) {
 	widget_class->measure = swt_fixed_measure;
 	widget_class->size_allocate = swt_fixed_size_allocate;
 
-	/* Accessibility implementation */
-	widget_class->get_accessible = swt_fixed_get_accessible;
+	signals[RESIZE] = g_signal_new(
+			"resize",
+			G_TYPE_FROM_CLASS (class),
+			G_SIGNAL_RUN_LAST,
+			0, NULL, NULL, NULL,
+	        G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
 }
 
 void swt_fixed_restack (SwtFixed *fixed, GtkWidget *widget, GtkWidget *sibling, gboolean above) {
@@ -775,17 +785,6 @@ void swt_fixed_restack (SwtFixed *fixed, GtkWidget *widget, GtkWidget *sibling,
 		list = above ? priv->children : NULL;
 	}
 	priv->children = g_list_insert_before (priv->children, list, child);
-
-	/*
-	{
-	GdkWindow *sibling_window = NULL;
-	if (list) {
-		child = list->data;
-		sibling_window = gtk_widget_get_window (child);
-	}
-	gdk_window_restack (gtk_widget_get_window (widget), sibling_window, above);
-	}
-	*/
 }
 
 static void swt_fixed_init (SwtFixed *widget) {
@@ -796,7 +795,7 @@ static void swt_fixed_init (SwtFixed *widget) {
 	priv->hadjustment = NULL;
 	priv->vadjustment = NULL;
 
-	gtk_widget_set_has_surface(GTK_WIDGET(widget), FALSE);
+//	gtk_widget_set_has_surface(GTK_WIDGET(widget), FALSE);
 }
 
 static void swt_fixed_finalize (GObject *object) {
@@ -805,7 +804,6 @@ static void swt_fixed_finalize (GObject *object) {
 
 	g_object_unref (priv->hadjustment);
 	g_object_unref (priv->vadjustment);
-	g_clear_object (&widget->accessible);
 
 	G_OBJECT_CLASS (swt_fixed_parent_class)->finalize (object);
 }
@@ -867,24 +865,6 @@ static void swt_fixed_set_property (GObject *object, guint prop_id, const GValue
     }
 }
 
-/*
-static void swt_fixed_realize (GtkWidget *widget) {
-	GtkAllocation allocation;
-	GdkSurface *surface;
-
- 	if (!gtk_widget_get_has_surface (widget)) {
-    	GTK_WIDGET_CLASS (swt_fixed_parent_class)->realize (widget);
-    	return;
-    }
-
-	gtk_widget_get_allocation (widget, &allocation);
-
-	surface = gdk_surface_new_child (gtk_widget_get_parent_surface (widget), &allocation);
-	gtk_widget_set_surface(widget, surface);
-	gdk_surface_set_user_data (surface, widget);
-	return GTK_WIDGET_CLASS (swt_fixed_parent_class)->realize (widget);
-}*/
-
 static void swt_fixed_map (GtkWidget *widget) {
 	SwtFixed *fixed = SWT_FIXED (widget);
 	SwtFixedPrivate *priv = fixed->priv;
@@ -899,99 +879,56 @@ static void swt_fixed_map (GtkWidget *widget) {
 			if (!gtk_widget_get_mapped (child)) gtk_widget_map (child);
 		}
 	}
-	if (gtk_widget_get_has_surface (widget)) {
-		//NOTE: contrary to most of GTK, swt_fixed_* container does not raise windows upon showing them.
-		//This has the effect that widgets are drawn *beneath* the previous one.
-		//E.g if this line is changed to gdk_window_show (..) then widgets are drawn on top of the previous one.
-		//This affects mostly only the absolute layout with overlapping widgets, e.g minimizied panels that
-		//pop-out in Eclipse (aka fast-view).
-		//As such, be attentive to swt_fixed_forall(..); traversing children may need to be done in reverse in some
-		//cases.
-		gdk_surface_show_unraised (gtk_widget_get_surface (widget));
-	}
+
+	//TODO: SWTFixed needs to show without raising. To have an effect that widgets are drawn beneath the previous one.
+
 	return GTK_WIDGET_CLASS (swt_fixed_parent_class)->map (widget);
 }
 
-/* Accessibility */
-static AtkObject *swt_fixed_get_accessible (GtkWidget *widget) {
-	SwtFixed *fixed = SWT_FIXED (widget);
-
-	if (!fixed->accessible) {
-		fixed->accessible = swt_fixed_accessible_new (widget);
-	}
-	return fixed->accessible;
-}
-
-static void swt_fixed_measure (GtkWidget *widget, GtkOrientation  orientation, int for_size, int *minimum,
-		int *natural, int *minimum_baseline, int *natural_baseline) {
-	SwtFixed *fixed = SWT_FIXED (widget);
-	SwtFixedPrivate *priv = fixed->priv;
-	GList *list;
-	int natural_size, child_nat;
-
-	list = priv->children;
-	natural_size = 0;
-
-	while (list) {
-		SwtFixedChild *child_data = list->data;
-		GtkWidget *child = child_data->widget;
+static void swt_fixed_measure (GtkWidget *widget, GtkOrientation  orientation, int for_size, int *minimum, int *natural, int *minimum_baseline, int *natural_baseline) {
+	for (GtkWidget* child = gtk_widget_get_first_child(widget); child != NULL; child = gtk_widget_get_next_sibling(child)) {
+		int child_nat = 0;
 
 		gtk_widget_measure(child, orientation, -1, NULL, &child_nat, NULL, NULL);
-		if (child_nat > natural_size) natural_size = child_nat;
-
-		list = list->next;
+		*natural = MAX(*natural, child_nat);
 	}
-	if (natural) *natural = natural_size;
+
 	if (minimum) *minimum = 0;
 	if (minimum_baseline) *minimum_baseline = -1;
 	if (natural_baseline) *natural_baseline = -1;
-	return;
 }
 
-static void swt_fixed_size_allocate (GtkWidget *widget, const GtkAllocation *allocation, int baseline) {
+static void swt_fixed_size_allocate (GtkWidget *widget, int width, int height, int baseline) {
+	g_signal_emit (widget, signals[RESIZE], 0, width, height);
+
 	SwtFixed *fixed = SWT_FIXED (widget);
 	SwtFixedPrivate *priv = fixed->priv;
-	GList *list;
-	GtkAllocation child_allocation;
-	GtkRequisition requisition;
-	gint w, h;
-
-	if (gtk_widget_get_has_surface (widget)) {
-		if (gtk_widget_get_realized (widget)) {
-			gdk_surface_move_resize (gtk_widget_get_surface (widget), allocation->x, allocation->y, allocation->width, allocation->height);
-	    }
-	}
-
-	list = priv->children;
+	GList* list = priv->children;
 
 	while (list) {
 		SwtFixedChild *child_data = list->data;
 		GtkWidget *child = child_data->widget;
-		list = list->next;
 
+		GtkAllocation child_allocation;
 		child_allocation.x = child_data->x;
 		child_allocation.y = child_data->y;
-		if (!gtk_widget_get_has_surface (widget)) {
-          child_allocation.x += allocation->x;
-          child_allocation.y += allocation->y;
-        }
 
-		w = child_data->width;
-		h = child_data->height;
+		int w = child_data->width;
+		int h = child_data->height;
 		if (w == -1 || h == -1) {
+			GtkRequisition requisition;
 			gtk_widget_get_preferred_size (child, &requisition, NULL);
 			if (w == -1) w = requisition.width;
 			if (h == -1) h = requisition.height;
 		}
-		// Feature in GTK: gtk_widget_preferred_size() has to be called before
-		// gtk_widget_size_allocate otherwise a warning is thrown. See Bug 486068.
-		gtk_widget_get_preferred_size (child, &requisition, NULL);
 
 		child_allocation.width = w;
 		child_allocation.height = h;
 
 		gtk_widget_size_allocate (child, &child_allocation, -1);
-    }
+
+		list = list->next;
+	}
 }
 
 void swt_fixed_move (SwtFixed *fixed, GtkWidget *widget, gint x, gint y) {
@@ -1049,17 +986,17 @@ void swt_fixed_resize (SwtFixed *fixed, GtkWidget *widget, gint width, gint heig
 }
 
 void swt_fixed_add (GtkWidget *container, GtkWidget *child) {
-	SwtFixed *fixed = SWT_FIXED (container);
+	SwtFixed *fixed = SWT_FIXED(container);
 	SwtFixedPrivate *priv = fixed->priv;
-	SwtFixedChild *child_data;
 
-	child_data = g_new (SwtFixedChild, 1);
+	SwtFixedChild *child_data = g_new(SwtFixedChild, 1);
 	child_data->widget = child;
   	child_data->x = child_data->y = 0;
   	child_data->width = child_data->height = -1;
 
-	priv->children = g_list_append (priv->children, child_data);
-	gtk_widget_set_parent (child, container);
+	priv->children = g_list_append(priv->children, child_data);
+
+	gtk_widget_set_parent(child, container);
 }
 
 void swt_fixed_remove (GtkWidget *container, GtkWidget *widget) {
@@ -1083,6 +1020,8 @@ void swt_fixed_remove (GtkWidget *container, GtkWidget *widget) {
 }
 
 #endif
+
+#if !defined(GTK4)
 static void swt_fixed_accessible_class_init (SwtFixedAccessibleClass *klass);
 static void swt_fixed_accessible_finalize (GObject *object);
 static void swt_fixed_accessible_initialize (AtkObject *obj, gpointer data);
@@ -1113,7 +1052,6 @@ struct _SwtFixedAccessiblePrivate {
 	GtkWidget *widget;
 };
 
-#if !defined(GTK4)
 G_DEFINE_TYPE_WITH_CODE (SwtFixedAccessible, swt_fixed_accessible, GTK_TYPE_CONTAINER_ACCESSIBLE,
 			 G_IMPLEMENT_INTERFACE (ATK_TYPE_ACTION, swt_fixed_accessible_action_iface_init)
 			 G_IMPLEMENT_INTERFACE (ATK_TYPE_COMPONENT, swt_fixed_accessible_component_iface_init)
@@ -1124,18 +1062,6 @@ G_DEFINE_TYPE_WITH_CODE (SwtFixedAccessible, swt_fixed_accessible, GTK_TYPE_CONT
 			 G_IMPLEMENT_INTERFACE (ATK_TYPE_TEXT, swt_fixed_accessible_text_iface_init)
 			 G_IMPLEMENT_INTERFACE (ATK_TYPE_VALUE, swt_fixed_accessible_value_iface_init)
 			 G_ADD_PRIVATE (SwtFixedAccessible))
-#else
-G_DEFINE_TYPE_WITH_CODE (SwtFixedAccessible, swt_fixed_accessible, GTK_TYPE_WIDGET_ACCESSIBLE,
-	 G_IMPLEMENT_INTERFACE (ATK_TYPE_ACTION, swt_fixed_accessible_action_iface_init)
-	 G_IMPLEMENT_INTERFACE (ATK_TYPE_COMPONENT, swt_fixed_accessible_component_iface_init)
-	 G_IMPLEMENT_INTERFACE (ATK_TYPE_EDITABLE_TEXT, swt_fixed_accessible_editable_text_iface_init)
-	 G_IMPLEMENT_INTERFACE (ATK_TYPE_HYPERTEXT, swt_fixed_accessible_hypertext_iface_init)
-	 G_IMPLEMENT_INTERFACE (ATK_TYPE_SELECTION, swt_fixed_accessible_selection_iface_init)
-	 G_IMPLEMENT_INTERFACE (ATK_TYPE_TABLE, swt_fixed_accessible_table_iface_init)
-	 G_IMPLEMENT_INTERFACE (ATK_TYPE_TEXT, swt_fixed_accessible_text_iface_init)
-	 G_IMPLEMENT_INTERFACE (ATK_TYPE_VALUE, swt_fixed_accessible_value_iface_init)
-	 G_ADD_PRIVATE (SwtFixedAccessible))
-#endif
 
 // Fully qualified Java class name for the Java implementation of ATK functions
 const char *ACCESSIBILITY_CLASS_NAME = "org/eclipse/swt/accessibility/AccessibleObject";
@@ -2244,7 +2170,6 @@ jlong call_accessible_object_function (const char *method_name, const char *meth
 	return result;
 }
 
-#if !defined(GTK4)
 //Add ability to debug gtk warnings for SWT snippets via SWT_FATAL_WARNINGS=1
 // env variable. Please see Eclipse bug 471477.
 // PLEASE NOTE: this functionality is only available on GTK3.
