@@ -323,9 +323,13 @@ void destroyWidget () {
 	releaseHandle ();
 	if (topHandle != 0 && (state & HANDLE) != 0) {
 		if ((style & SWT.BALLOON) != 0) {
-			GTK.gtk_widget_destroy (topHandle);
+			if (GTK.GTK4) {
+				OS.g_object_unref(topHandle);
+			} else {
+				GTK.gtk_widget_destroy(topHandle);
+			}
 		} else {
-			OS.g_object_unref (topHandle);
+			OS.g_object_unref(topHandle);
 		}
 	}
 }
@@ -350,6 +354,7 @@ public boolean getAutoHide () {
 Point getLocation () {
 	int x = this.x;
 	int y = this.y;
+
 	if (item != null) {
 		long itemHandle = item.handle;
 		GdkRectangle area = new GdkRectangle ();
@@ -357,20 +362,24 @@ Point getLocation () {
 		x = area.x + area.width / 2;
 		y = area.y + area.height / 2;
 	}
+
 	if (x == -1 || y == -1) {
-		int [] px = new int [1], py = new int [1];
 		if (GTK.GTK4) {
-			/*
-			 * TODO: calling gdk_window_get_device_position() with a 0
-			 * for the GdkWindow uses gdk_get_default_root_window(),
-			 * which doesn't exist on GTK4.
-			 */
+			double[] px = new double[1], py = new double[1];
+
+			long surface = GTK.gtk_native_get_surface(GTK.gtk_widget_get_native(handle));
+			display.getSurfacePointerPosition(surface, px, py, null);
+			x = (int)px[0];
+			y = (int)py[0];
 		} else {
-			display.gdk_window_get_device_position (0, px, py, null);
+			int[] px = new int[1], py = new int[1];
+
+			display.getWindowPointerPosition (0, px, py, null);
+			x = px[0];
+			y = py[0];
 		}
-		x = px [0];
-		y = py [0];
 	}
+
 	return new Point(x, y);
 }
 
@@ -482,21 +491,6 @@ public boolean getVisible () {
 	return false;
 }
 
-@Override
-long gtk_event (long widget, long event) {
-	if (!GTK.GTK4) return 0;
-	int eventType = GDK.gdk_event_get_event_type(event);
-	switch (eventType) {
-		case GDK.GDK4_BUTTON_PRESS: {
-			return gtk_button_press_event(widget, event);
-		}
-		case GDK.GDK4_BUTTON_RELEASE: {
-			return gtk_button_release_event(widget, event);
-		}
-	}
-	return 0;
-}
-
 @Override
 long gtk_button_press_event (long widget, long event) {
 	sendSelectionEvent (SWT.Selection, null, true);
@@ -597,11 +591,14 @@ void hookEvents () {
 	if ((style & SWT.BALLOON) != 0) {
 		OS.g_signal_connect_closure_by_id (handle, display.signalIds [EXPOSE_EVENT], 0, display.getClosure (EXPOSE_EVENT), true);
 		OS.g_signal_connect_closure_by_id (handle, display.signalIds [EXPOSE_EVENT_INVERSE], 0, display.getClosure (EXPOSE_EVENT_INVERSE), true);
-		GTK.gtk_widget_add_events (handle, GDK.GDK_BUTTON_PRESS_MASK);
+
 		if (GTK.GTK4) {
-			OS.g_signal_connect_closure_by_id (handle, display.signalIds [EVENT], 0, display.getClosure (EVENT), false);
+			long clickController = GTK.gtk_gesture_click_new();
+			GTK.gtk_widget_add_controller(handle, clickController);
+			OS.g_signal_connect(clickController, OS.pressed, display.gesturePressReleaseProc, GESTURE_PRESSED);
 		} else {
-			OS.g_signal_connect_closure (handle, OS.button_press_event, display.getClosure (BUTTON_PRESS_EVENT), false);
+			GTK.gtk_widget_add_events(handle, GDK.GDK_BUTTON_PRESS_MASK);
+			OS.g_signal_connect_closure(handle, OS.button_press_event, display.getClosure (BUTTON_PRESS_EVENT), false);
 		}
 	}
 }
