@@ -31,7 +31,7 @@ import org.eclipse.swt.internal.win32.*;
  * <em>dialog</em> shells.
  * <p>
  * Instances are always displayed in one of the maximized,
- * minimized or normal states:
+ * minimized or normal states:</p>
  * <ul>
  * <li>
  * When an instance is marked as <em>maximized</em>, the
@@ -55,7 +55,7 @@ import org.eclipse.swt.internal.win32.*;
  * of these changes are also possible.
  * </li>
  * </ul>
- * </p><p>
+ * <p>
  * The <em>modality</em> of an instance may be specified using
  * style bits. The modality style bits are used to determine
  * whether input is blocked for other shells on the display.
@@ -81,7 +81,7 @@ import org.eclipse.swt.internal.win32.*;
  * restrictive style. For example, most operating systems
  * no longer support <code>SYSTEM_MODAL</code> because
  * it can freeze up the desktop, so this is typically
- * downgraded to <code>APPLICATION_MODAL</code>.
+ * downgraded to <code>APPLICATION_MODAL</code>.</p>
  * <dl>
  * <dt><b>Styles:</b></dt>
  * <dd>BORDER, CLOSE, MIN, MAX, NO_MOVE, NO_TRIM, RESIZE, TITLE, ON_TOP, TOOL, SHEET</dd>
@@ -89,8 +89,9 @@ import org.eclipse.swt.internal.win32.*;
  * <dt><b>Events:</b></dt>
  * <dd>Activate, Close, Deactivate, Deiconify, Iconify</dd>
  * </dl>
+ * <p>
  * Class <code>SWT</code> provides two "convenience constants"
- * for the most commonly required style combinations:
+ * for the most commonly required style combinations:</p>
  * <dl>
  * <dt><code>SHELL_TRIM</code></dt>
  * <dd>
@@ -105,7 +106,6 @@ import org.eclipse.swt.internal.win32.*;
  * is, <code>TITLE | CLOSE | BORDER</code>)
  * </dd>
  * </dl>
- * </p>
  * <p>
  * Note: Only one of the styles APPLICATION_MODAL, MODELESS,
  * PRIMARY_MODAL and SYSTEM_MODAL may be specified.
@@ -123,16 +123,16 @@ import org.eclipse.swt.internal.win32.*;
 public class Shell extends Decorations {
 	Menu activeMenu;
 	ToolTip [] toolTips;
-	long /*int*/ hIMC, hwndMDIClient, lpstrTip, toolTipHandle, balloonTipHandle, menuItemToolTipHandle;
+	long hwndMDIClient, lpstrTip, toolTipHandle, balloonTipHandle, menuItemToolTipHandle;
 	int minWidth = SWT.DEFAULT, minHeight = SWT.DEFAULT;
-	long /*int*/ [] brushes;
+	long [] brushes;
 	boolean showWithParent, fullScreen, wasMaximized, modified, center;
 	String toolTitle, balloonTitle;
-	long /*int*/ toolIcon, balloonIcon;
-	long /*int*/ windowProc;
+	long toolIcon, balloonIcon;
+	long windowProc;
 	Control lastActive;
-	static /*final*/ long /*int*/ ToolTipProc;
-	static final long /*int*/ DialogProc;
+	static /*final*/ long ToolTipProc;
+	static final long DialogProc;
 	static final TCHAR DialogClass = new TCHAR (0, "#32770", true);
 	final static int [] SYSTEM_COLORS = {
 		OS.COLOR_BTNFACE,
@@ -279,7 +279,7 @@ public Shell (Display display, int style) {
 	this (display, null, style, 0, false);
 }
 
-Shell (Display display, Shell parent, int style, long /*int*/ handle, boolean embedded) {
+Shell (Display display, Shell parent, int style, long handle, boolean embedded) {
 	super ();
 	checkSubclass ();
 	if (display == null) display = Display.getCurrent ();
@@ -398,7 +398,7 @@ public Shell (Shell parent, int style) {
  *
  * @noreference This method is not intended to be referenced by clients.
  */
-public static Shell win32_new (Display display, long /*int*/ handle) {
+public static Shell win32_new (Display display, long handle) {
 	return new Shell (display, null, SWT.NO_TRIM, handle, true);
 }
 
@@ -421,7 +421,7 @@ public static Shell win32_new (Display display, long /*int*/ handle) {
  *
  * @since 3.3
  */
-public static Shell internal_new (Display display, long /*int*/ handle) {
+public static Shell internal_new (Display display, long handle) {
 	return new Shell (display, null, SWT.NO_TRIM, handle, false);
 }
 
@@ -473,13 +473,13 @@ public void addShellListener (ShellListener listener) {
 	addListener (SWT.Deactivate, typedListener);
 }
 
-long /*int*/ balloonTipHandle () {
+long balloonTipHandle () {
 	if (balloonTipHandle == 0) createBalloonTipHandle ();
 	return balloonTipHandle;
 }
 
 @Override
-long /*int*/ callWindowProc (long /*int*/ hwnd, int msg, long /*int*/ wParam, long /*int*/ lParam) {
+long callWindowProc (long hwnd, int msg, long wParam, long lParam) {
 	if (handle == 0) return 0;
 	if (hwnd == toolTipHandle || hwnd == balloonTipHandle || hwnd == menuItemToolTipHandle) {
 		return OS.CallWindowProc (ToolTipProc, hwnd, msg, wParam, lParam);
@@ -625,10 +625,6 @@ void createHandle () {
 		int flags = OS.SWP_DRAWFRAME | OS.SWP_NOMOVE | OS.SWP_NOSIZE | OS.SWP_NOZORDER | OS.SWP_NOACTIVATE;
 		OS.SetWindowPos (handle, 0, 0, 0, 0, 0, flags);
 	}
-	if (OS.IsDBLocale) {
-		hIMC = OS.ImmCreateContext ();
-		if (hIMC != 0) OS.ImmAssociateContext (handle, hIMC);
-	}
 }
 
 void createMenuItemToolTipHandle() {
@@ -659,8 +655,8 @@ void createToolTipHandle () {
 	toolTipHandle = createToolTipHandle (handle);
 }
 
-long /*int*/ createToolTipHandle (long /*int*/ parent) {
-	long /*int*/ toolTipHandle = OS.CreateWindowEx (
+long createToolTipHandle (long parent) {
+	long toolTipHandle = OS.CreateWindowEx (
 		0,
 		new TCHAR (0, OS.TOOLTIPS_CLASS, true),
 		null,
@@ -712,36 +708,6 @@ void destroyToolTip (ToolTip toolTip) {
 void destroyWidget () {
 	fixActiveShell ();
 	super.destroyWidget ();
-
-	/*
-	* Destroy context only after the controls that used it were destroyed.
-	* Technically, that shouldn't be necessary, because 'Control.releaseWidget'
-	* clears up association by calling 'OS.ImmAssociateContext (handle, 0)'.
-	* However, there's a bug in Windows 10 (see bug 526758), and this is the workaround.
-	*/
-	if (OS.IsDBLocale) {
-		if (hIMC != 0) OS.ImmDestroyContext (hIMC);
-	}
-}
-
-@Override
-public void dispose () {
-	/*
-	* This code is intentionally commented.  On some
-	* platforms, the owner window is repainted right
-	* away when a dialog window exits.  This behavior
-	* is currently unspecified.
-	*/
-//	/*
-//	* Note:  It is valid to attempt to dispose a widget
-//	* more than once.  If this happens, fail silently.
-//	*/
-//	if (!isValidWidget ()) return;
-//	if (!isValidThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
-//	Display oldDisplay = display;
-	super.dispose ();
-	// widget is disposed at this point
-//	if (oldDisplay != null) oldDisplay.update ();
 }
 
 @Override
@@ -759,18 +725,17 @@ void enableWidget (boolean enabled) {
 }
 
 @Override
-long /*int*/ findBrush (long /*int*/ value, int lbStyle) {
+long findBrush (long value, int lbStyle) {
 	if (lbStyle == OS.BS_SOLID) {
-		for (int i=0; i<SYSTEM_COLORS.length; i++) {
-			if (value == OS.GetSysColor (SYSTEM_COLORS [i])) {
-				return OS.GetSysColorBrush (SYSTEM_COLORS [i]);
+		for (int element : SYSTEM_COLORS) {
+			if (value == OS.GetSysColor (element)) {
+				return OS.GetSysColorBrush (element);
 			}
 		}
 	}
-	if (brushes == null) brushes = new long /*int*/ [BRUSHES_SIZE];
+	if (brushes == null) brushes = new long [BRUSHES_SIZE];
 	LOGBRUSH logBrush = new LOGBRUSH ();
-	for (int i=0; i<brushes.length; i++) {
-		long /*int*/ hBrush = brushes [i];
+	for (long hBrush : brushes) {
 		if (hBrush == 0) break;
 		OS.GetObject (hBrush, LOGBRUSH.sizeof, logBrush);
 		switch (logBrush.lbStyle) {
@@ -787,12 +752,12 @@ long /*int*/ findBrush (long /*int*/ value, int lbStyle) {
 		}
 	}
 	int length = brushes.length;
-	long /*int*/ hBrush = brushes [--length];
+	long hBrush = brushes [--length];
 	if (hBrush != 0) OS.DeleteObject (hBrush);
 	System.arraycopy (brushes, 0, brushes, 1, length);
 	switch (lbStyle) {
 		case OS.BS_SOLID:
-			hBrush = OS.CreateSolidBrush ((int)/*64*/value);
+			hBrush = OS.CreateSolidBrush ((int)value);
 			break;
 		case OS.BS_PATTERN:
 			hBrush = OS.CreatePatternBrush (value);
@@ -832,7 +797,7 @@ void fixActiveShell () {
 	* application).  The fix is to activate the disabled parent
 	* shell before disposing or hiding the active shell.
 	*/
-	long /*int*/ hwndParent = OS.GetParent (handle);
+	long hwndParent = OS.GetParent (handle);
 	if (hwndParent != 0 && handle == OS.GetActiveWindow ()) {
 		if (!OS.IsWindowEnabled (hwndParent) && OS.IsWindowVisible (hwndParent)) {
 			OS.SetActiveWindow (hwndParent);
@@ -960,13 +925,13 @@ ToolTip getCurrentToolTip () {
 	return null;
 }
 
-ToolTip getCurrentToolTip (long /*int*/ hwndToolTip) {
+ToolTip getCurrentToolTip (long hwndToolTip) {
 	if (hwndToolTip == 0) return null;
 	if (OS.SendMessage (hwndToolTip, OS.TTM_GETCURRENTTOOL, 0, 0) != 0) {
 		TOOLINFO lpti = new TOOLINFO ();
 		lpti.cbSize = TOOLINFO.sizeof;
 		if (OS.SendMessage (hwndToolTip, OS.TTM_GETCURRENTTOOL, 0, lpti) != 0) {
-			if ((lpti.uFlags & OS.TTF_IDISHWND) == 0) return findToolTip ((int)/*64*/lpti.uId);
+			if ((lpti.uFlags & OS.TTF_IDISHWND) == 0) return findToolTip ((int)lpti.uId);
 		}
 	}
 	return null;
@@ -1017,7 +982,7 @@ public boolean getFullScreen () {
 public int getImeInputMode () {
 	checkWidget ();
 	if (!OS.IsDBLocale) return 0;
-	long /*int*/ hIMC = OS.ImmGetContext (handle);
+	long hIMC = OS.ImmGetContext (handle);
 	int [] lpfdwConversion = new int [1], lpfdwSentence = new int [1];
 	boolean open = OS.ImmGetOpenStatus (hIMC);
 	if (open) open = OS.ImmGetConversionStatus (hIMC, lpfdwConversion, lpfdwSentence);
@@ -1153,22 +1118,22 @@ public Shell [] getShells () {
 	checkWidget ();
 	int count = 0;
 	Shell [] shells = display.getShells ();
-	for (int i=0; i<shells.length; i++) {
-		Control shell = shells [i];
+	for (Shell activeshell : shells) {
+		Control shell = activeshell;
 		do {
-			shell = shell.parent;
+			shell = shell.getParent ();
 		} while (shell != null && shell != this);
 		if (shell == this) count++;
 	}
 	int index = 0;
 	Shell [] result = new Shell [count];
-	for (int i=0; i<shells.length; i++) {
-		Control shell = shells [i];
+	for (Shell activeshell : shells) {
+		Control shell = activeshell;
 		do {
-			shell = shell.parent;
+			shell = shell.getParent ();
 		} while (shell != null && shell != this);
 		if (shell == this) {
-			result [index++] = shells [i];
+			result [index++] = activeshell;
 		}
 	}
 	return result;
@@ -1212,7 +1177,7 @@ public boolean isVisible () {
 	return getVisible ();
 }
 
-long /*int*/ hwndMDIClient () {
+long hwndMDIClient () {
 	if (hwndMDIClient == 0) {
 		int widgetStyle = OS.MDIS_ALLCHILDSTYLES | OS.WS_CHILD | OS.WS_CLIPCHILDREN | OS.WS_CLIPSIBLINGS;
 		hwndMDIClient = OS.CreateWindowEx (
@@ -1230,7 +1195,7 @@ long /*int*/ hwndMDIClient () {
 	return hwndMDIClient;
 }
 
-long /*int*/ menuItemToolTipHandle () {
+long menuItemToolTipHandle () {
 	if (menuItemToolTipHandle == 0) createMenuItemToolTipHandle ();
 	return menuItemToolTipHandle;
 }
@@ -1325,8 +1290,8 @@ void register () {
 
 void releaseBrushes () {
 	if (brushes != null) {
-		for (int i=0; i<brushes.length; i++) {
-			if (brushes [i] != 0) OS.DeleteObject (brushes [i]);
+		for (long brush : brushes) {
+			if (brush != 0) OS.DeleteObject (brush);
 		}
 	}
 	brushes = null;
@@ -1334,16 +1299,13 @@ void releaseBrushes () {
 
 @Override
 void releaseChildren (boolean destroy) {
-	Shell [] shells = getShells ();
-	for (int i=0; i<shells.length; i++) {
-		Shell shell = shells [i];
+	for (Shell shell : getShells ()) {
 		if (shell != null && !shell.isDisposed ()) {
 			shell.release (false);
 		}
 	}
 	if (toolTips != null) {
-		for (int i=0; i<toolTips.length; i++) {
-			ToolTip toolTip = toolTips [i];
+		for (ToolTip toolTip : toolTips) {
 			if (toolTip != null && !toolTip.isDisposed ()) {
 				toolTip.release (false);
 			}
@@ -1371,7 +1333,7 @@ void releaseWidget () {
 	activeMenu = null;
 	display.clearModal (this);
 	if (lpstrTip != 0) {
-		long /*int*/ hHeap = OS.GetProcessHeap ();
+		long hHeap = OS.GetProcessHeap ();
 		OS.HeapFree (hHeap, 0, lpstrTip);
 	}
 	lpstrTip = 0;
@@ -1421,36 +1383,19 @@ public void requestLayout () {
 
 @Override
 void reskinChildren (int flags) {
-	Shell [] shells = getShells ();
-	for (int i=0; i<shells.length; i++) {
-		Shell shell = shells [i];
+	for (Shell shell : getShells ()) {
 		if (shell != null) shell.reskin (flags);
 	}
 	if (toolTips != null) {
-		for (int i=0; i<toolTips.length; i++) {
-			ToolTip toolTip = toolTips [i];
+		for (ToolTip toolTip : toolTips) {
 			if (toolTip != null) toolTip.reskin (flags);
 		}
 	}
 	super.reskinChildren (flags);
 }
 
-LRESULT selectPalette (long /*int*/ hPalette) {
-	long /*int*/ hDC = OS.GetDC (handle);
-	long /*int*/ hOld = OS.SelectPalette (hDC, hPalette, false);
-	int result = OS.RealizePalette (hDC);
-	if (result > 0) {
-		OS.InvalidateRect (handle, null, true);
-	} else {
-		OS.SelectPalette (hDC, hOld, true);
-		OS.RealizePalette (hDC);
-	}
-	OS.ReleaseDC (handle, hDC);
-	return (result > 0) ? LRESULT.ONE : LRESULT.ZERO;
-}
-
 @Override
-boolean sendKeyEvent (int type, int msg, long /*int*/ wParam, long /*int*/ lParam, Event event) {
+boolean sendKeyEvent (int type, int msg, long wParam, long lParam, Event event) {
 	if (!isEnabled () || !isActive ()) return false;
 	return super.sendKeyEvent (type, msg, wParam, lParam, event);
 }
@@ -1626,6 +1571,9 @@ public void setFullScreen (boolean fullScreen) {
 	}
 	if (fullScreen) wasMaximized = getMaximized ();
 	boolean visible = isVisible ();
+	if (!visible && !wasMaximized) {
+		swFlags = stateFlags;
+	}
 	OS.SetWindowLong (handle, OS.GWL_STYLE, styleFlags);
 	if (wasMaximized) {
 		OS.ShowWindow (handle, OS.SW_HIDE);
@@ -1656,7 +1604,7 @@ public void setImeInputMode (int mode) {
 	checkWidget ();
 	if (!OS.IsDBLocale) return;
 	boolean imeOn = mode != SWT.NONE;
-	long /*int*/ hIMC = OS.ImmGetContext (handle);
+	long hIMC = OS.ImmGetContext (handle);
 	OS.ImmSetOpenStatus (hIMC, imeOn);
 	if (imeOn) {
 		int [] lpfdwConversion = new int [1], lpfdwSentence = new int [1];
@@ -1674,7 +1622,7 @@ public void setImeInputMode (int mode) {
 			}
 			boolean fullShape = (mode & SWT.DBCS) != 0;
 			if ((mode & SWT.NATIVE) != 0) {
-				long /*int*/ hkl = OS.GetKeyboardLayout (0);
+				long hkl = OS.GetKeyboardLayout (0);
 				int langid = OS.PRIMARYLANGID (OS.LOWORD (hkl));
 				if (langid == OS.LANG_JAPANESE) {
 					fullShape = true;
@@ -1772,7 +1720,6 @@ public void setMinimumSize (Point size) {
  *
  * @param modified the new modified state for the receiver
  *
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -1786,7 +1733,7 @@ public void setModified (boolean modified) {
 }
 
 void setItemEnabled (int cmd, boolean enabled) {
-	long /*int*/ hMenu = OS.GetSystemMenu (handle, false);
+	long hMenu = OS.GetSystemMenu (handle, false);
 	if (hMenu == 0) return;
 	int flags = OS.MF_ENABLED;
 	if (!enabled) flags = OS.MF_DISABLED | OS.MF_GRAYED;
@@ -1833,12 +1780,12 @@ public void setRegion (Region region) {
 	super.setRegion (region);
 }
 
-void setToolTipText (long /*int*/ hwnd, String text) {
+void setToolTipText (long hwnd, String text) {
 	TOOLINFO lpti = new TOOLINFO ();
 	lpti.cbSize = TOOLINFO.sizeof;
 	lpti.hwnd = handle;
 	lpti.uId = hwnd;
-	long /*int*/ hwndToolTip = toolTipHandle ();
+	long hwndToolTip = toolTipHandle ();
 	if (text == null) {
 		OS.SendMessage (hwndToolTip, OS.TTM_DELTOOL, 0, lpti);
 	} else {
@@ -1860,7 +1807,7 @@ void setToolTipText (NMTTDISPINFO lpnmtdi, char [] buffer) {
 	* shell trimmings.
 	*/
 	if (!hasCursor ()) return;
-	long /*int*/ hHeap = OS.GetProcessHeap ();
+	long hHeap = OS.GetProcessHeap ();
 	if (lpstrTip != 0) OS.HeapFree (hHeap, 0, lpstrTip);
 	int byteCount = buffer.length * 2;
 	lpstrTip = OS.HeapAlloc (hHeap, OS.HEAP_ZERO_MEMORY, byteCount);
@@ -1868,7 +1815,7 @@ void setToolTipText (NMTTDISPINFO lpnmtdi, char [] buffer) {
 	lpnmtdi.lpszText = lpstrTip;
 }
 
-void setToolTipTitle (long /*int*/ hwndToolTip, String text, int icon) {
+void setToolTipTitle (long hwndToolTip, String text, int icon) {
 	/*
 	* Bug in Windows.  For some reason, when TTM_SETTITLE
 	* is used to set the title of a tool tip, Windows leaks
@@ -1941,7 +1888,7 @@ public void setVisible (boolean visible) {
 				bringToTop ();
 				if (isDisposed ()) return;
 			}
-			long /*int*/ hwndShell = OS.GetActiveWindow ();
+			long hwndShell = OS.GetActiveWindow ();
 			if (hwndShell == 0) {
 				if (parent != null) hwndShell = parent.handle;
 			}
@@ -1982,7 +1929,7 @@ public void setVisible (boolean visible) {
 	/* Make the foreign window parent appear in the task bar */
 	if (visible) {
 		if (parent != null && (parent.state & FOREIGN_HANDLE) != 0) {
-			long /*int*/ hwndParent = parent.handle;
+			long hwndParent = parent.handle;
 			int style = OS.GetWindowLong (hwndParent, OS.GWL_EXSTYLE);
 			if ((style & OS.WS_EX_TOOLWINDOW) != 0) {
 				OS.SetWindowLong (hwndParent, OS.GWL_EXSTYLE, style & ~OS.WS_EX_TOOLWINDOW);
@@ -2002,7 +1949,7 @@ public void setVisible (boolean visible) {
 void subclass () {
 	super.subclass ();
 	if (ToolTipProc != 0) {
-		long /*int*/ newProc = display.windowProc;
+		long newProc = display.windowProc;
 		if (toolTipHandle != 0) {
 			OS.SetWindowLongPtr (toolTipHandle, OS.GWLP_WNDPROC, newProc);
 		}
@@ -2015,7 +1962,7 @@ void subclass () {
 	}
 }
 
-long /*int*/ toolTipHandle () {
+long toolTipHandle () {
 	if (toolTipHandle == 0) createToolTipHandle ();
 	return toolTipHandle;
 }
@@ -2065,7 +2012,7 @@ CREATESTRUCT widgetCreateStruct () {
 }
 
 @Override
-long /*int*/ widgetParent () {
+long widgetParent () {
 	if (handle != 0) return handle;
 	return parent != null ? parent.handle : 0;
 }
@@ -2104,7 +2051,7 @@ TCHAR windowClass () {
 }
 
 @Override
-long /*int*/ windowProc () {
+long windowProc () {
 	if (windowProc != 0) return windowProc;
 	if ((style & SWT.TOOL) != 0) {
 		int trim = SWT.TITLE | SWT.CLOSE | SWT.MIN | SWT.MAX | SWT.BORDER | SWT.RESIZE;
@@ -2113,8 +2060,39 @@ long /*int*/ windowProc () {
 	return parent != null ? DialogProc : super.windowProc ();
 }
 
+Rectangle getClientRectInWindow () {
+	RECT windowRect = new RECT ();
+	OS.GetWindowRect (handle, windowRect);
+
+	POINT clientWindowLT = new POINT ();
+	OS.ClientToScreen (handle, clientWindowLT);
+	clientWindowLT.x -= windowRect.left;
+	clientWindowLT.y -= windowRect.top;
+
+	RECT clientRect = new RECT ();
+	OS.GetClientRect (handle, clientRect);
+
+	return new Rectangle(
+			clientWindowLT.x + clientRect.left,
+			clientWindowLT.y + clientRect.top,
+			clientRect.right - clientRect.left,
+			clientRect.bottom - clientRect.top);
+}
+
+void overpaintMenuBorder () {
+	if ((menuBar == null) || (display.menuBarBorderPen == 0)) return;
+
+	Rectangle clientArea = getClientRectInWindow();
+	long dc = OS.GetWindowDC (handle);
+	long oldPen = OS.SelectObject (dc, display.menuBarBorderPen);
+	OS.MoveToEx (dc, clientArea.x, clientArea.y - 1, 0);
+	OS.LineTo (dc, clientArea.x + clientArea.width, clientArea.y - 1);
+	OS.SelectObject (dc, oldPen);
+	OS.ReleaseDC (handle, dc);
+}
+
 @Override
-long /*int*/ windowProc (long /*int*/ hwnd, int msg, long /*int*/ wParam, long /*int*/ lParam) {
+long windowProc (long hwnd, int msg, long wParam, long lParam) {
 	if (handle == 0) return 0;
 	if((style & SWT.NO_MOVE) != 0 && msg == OS.WM_NCLBUTTONDOWN && wParam == OS.HTCAPTION) return 0;
 	if (hwnd == toolTipHandle || hwnd == balloonTipHandle || hwnd == menuItemToolTipHandle) {
@@ -2139,7 +2117,7 @@ long /*int*/ windowProc (long /*int*/ hwnd, int msg, long /*int*/ wParam, long /
 		return callWindowProc (hwnd, msg, wParam, lParam);
 	}
 	if (hwnd == handle) {
-		if ((int)/*64*/msg == Display.TASKBARBUTTONCREATED) {
+		if (msg == Display.TASKBARBUTTONCREATED) {
 			if (display.taskBar != null) {
 				for (TaskItem item : display.taskBar.items) {
 					if (item != null && item.shell == this) {
@@ -2150,6 +2128,17 @@ long /*int*/ windowProc (long /*int*/ hwnd, int msg, long /*int*/ wParam, long /
 			}
 		}
 	}
+
+	switch (msg) {
+		case OS.WM_NCACTIVATE:
+		case OS.WM_NCPAINT:
+		{
+			long ret = super.windowProc (hwnd, msg, wParam, lParam);
+			overpaintMenuBorder();
+			return ret;
+		}
+	}
+
 	return super.windowProc (hwnd, msg, wParam, lParam);
 }
 
@@ -2176,19 +2165,7 @@ int widgetStyle () {
 }
 
 @Override
-LRESULT WM_ACTIVATE (long /*int*/ wParam, long /*int*/ lParam) {
-	/*
-	* Bug in Windows XP.  When a Shell is deactivated, the
-	* IME composition window does not go away. This causes
-	* repaint issues.  The fix is to commit the composition
-	* string.
-	*/
-	if (OS.LOWORD (wParam) == 0 && OS.IsDBLocale && hIMC != 0) {
-		if (OS.ImmGetOpenStatus (hIMC)) {
-			OS.ImmNotifyIME (hIMC, OS.NI_COMPOSITIONSTR, OS.CPS_COMPLETE, 0);
-		}
-	}
-
+LRESULT WM_ACTIVATE (long wParam, long lParam) {
 	/* Process WM_ACTIVATE */
 	LRESULT result = super.WM_ACTIVATE (wParam, lParam);
 	if (OS.LOWORD (wParam) == 0) {
@@ -2202,7 +2179,7 @@ LRESULT WM_ACTIVATE (long /*int*/ wParam, long /*int*/ lParam) {
 }
 
 @Override
-LRESULT WM_DESTROY (long /*int*/ wParam, long /*int*/ lParam) {
+LRESULT WM_DESTROY (long wParam, long lParam) {
 	LRESULT result = super.WM_DESTROY (wParam, lParam);
 	/*
 	* When the shell is a WS_CHILD window of a non-SWT
@@ -2219,7 +2196,7 @@ LRESULT WM_DESTROY (long /*int*/ wParam, long /*int*/ lParam) {
 }
 
 @Override
-LRESULT WM_ERASEBKGND (long /*int*/ wParam, long /*int*/ lParam) {
+LRESULT WM_ERASEBKGND (long wParam, long lParam) {
 	LRESULT result = super.WM_ERASEBKGND (wParam, lParam);
 	if (result != null) return result;
 	/*
@@ -2238,7 +2215,7 @@ LRESULT WM_ERASEBKGND (long /*int*/ wParam, long /*int*/ lParam) {
 }
 
 @Override
-LRESULT WM_ENTERIDLE (long /*int*/ wParam, long /*int*/ lParam) {
+LRESULT WM_ENTERIDLE (long wParam, long lParam) {
 	LRESULT result = super.WM_ENTERIDLE (wParam, lParam);
 	if (result != null) return result;
 	Display display = this.display;
@@ -2247,7 +2224,7 @@ LRESULT WM_ENTERIDLE (long /*int*/ wParam, long /*int*/ lParam) {
 }
 
 @Override
-LRESULT WM_GETMINMAXINFO (long /*int*/ wParam, long /*int*/ lParam) {
+LRESULT WM_GETMINMAXINFO (long wParam, long lParam) {
 	LRESULT result = super.WM_GETMINMAXINFO (wParam, lParam);
 	if (result != null) return result;
 	if (minWidth != SWT.DEFAULT || minHeight != SWT.DEFAULT) {
@@ -2262,7 +2239,7 @@ LRESULT WM_GETMINMAXINFO (long /*int*/ wParam, long /*int*/ lParam) {
 }
 
 @Override
-LRESULT WM_MOUSEACTIVATE (long /*int*/ wParam, long /*int*/ lParam) {
+LRESULT WM_MOUSEACTIVATE (long wParam, long lParam) {
 	LRESULT result = super.WM_MOUSEACTIVATE (wParam, lParam);
 	if (result != null) return result;
 
@@ -2313,7 +2290,7 @@ LRESULT WM_MOUSEACTIVATE (long /*int*/ wParam, long /*int*/ lParam) {
 		int pos = OS.GetMessagePos ();
 		OS.POINTSTOPOINT (pt, pos);
 	}
-	long /*int*/ hwnd = OS.WindowFromPoint (pt);
+	long hwnd = OS.WindowFromPoint (pt);
 	if (hwnd == 0) return null;
 	Control control = display.findControl (hwnd);
 
@@ -2334,13 +2311,13 @@ LRESULT WM_MOUSEACTIVATE (long /*int*/ wParam, long /*int*/ lParam) {
 		}
 	}
 
-	long /*int*/ code = callWindowProc (handle, OS.WM_MOUSEACTIVATE, wParam, lParam);
+	long code = callWindowProc (handle, OS.WM_MOUSEACTIVATE, wParam, lParam);
 	setActiveControl (control, SWT.MouseDown);
 	return new LRESULT (code);
 }
 
 @Override
-LRESULT WM_MOVE (long /*int*/ wParam, long /*int*/ lParam) {
+LRESULT WM_MOVE (long wParam, long lParam) {
 	LRESULT result = super.WM_MOVE (wParam, lParam);
 	if (result != null) return result;
 	ToolTip tip = getCurrentToolTip ();
@@ -2349,16 +2326,16 @@ LRESULT WM_MOVE (long /*int*/ wParam, long /*int*/ lParam) {
 }
 
 @Override
-LRESULT WM_NCHITTEST (long /*int*/ wParam, long /*int*/ lParam) {
+LRESULT WM_NCHITTEST (long wParam, long lParam) {
 	if (!OS.IsWindowEnabled (handle)) return null;
 	if (!isEnabled () || !isActive ()) {
 		if (!Display.TrimEnabled) return new LRESULT (OS.HTNOWHERE);
-		long /*int*/ hittest = callWindowProc (handle, OS.WM_NCHITTEST, wParam, lParam);
+		long hittest = callWindowProc (handle, OS.WM_NCHITTEST, wParam, lParam);
 		if (hittest == OS.HTCLIENT || hittest == OS.HTMENU) hittest = OS.HTBORDER;
 		return new LRESULT (hittest);
 	}
 	if (menuBar != null && !menuBar.getEnabled ()) {
-		long /*int*/ hittest = callWindowProc (handle, OS.WM_NCHITTEST, wParam, lParam);
+		long hittest = callWindowProc (handle, OS.WM_NCHITTEST, wParam, lParam);
 		if (hittest == OS.HTMENU) hittest = OS.HTBORDER;
 		return new LRESULT (hittest);
 	}
@@ -2366,7 +2343,7 @@ LRESULT WM_NCHITTEST (long /*int*/ wParam, long /*int*/ lParam) {
 }
 
 @Override
-LRESULT WM_NCLBUTTONDOWN (long /*int*/ wParam, long /*int*/ lParam) {
+LRESULT WM_NCLBUTTONDOWN (long wParam, long lParam) {
 	LRESULT result = super.WM_NCLBUTTONDOWN (wParam, lParam);
 	if (result != null) return result;
 	/*
@@ -2379,7 +2356,7 @@ LRESULT WM_NCLBUTTONDOWN (long /*int*/ wParam, long /*int*/ lParam) {
 	if (!display.ignoreRestoreFocus) return result;
 	Display display = this.display;
 	display.lockActiveWindow = true;
-	long /*int*/ code = callWindowProc (handle, OS.WM_NCLBUTTONDOWN, wParam, lParam);
+	long code = callWindowProc (handle, OS.WM_NCLBUTTONDOWN, wParam, lParam);
 	display.lockActiveWindow = false;
 	Control focusControl = display.lastHittestControl;
 	if (focusControl != null && !focusControl.isDisposed ()) {
@@ -2391,23 +2368,7 @@ LRESULT WM_NCLBUTTONDOWN (long /*int*/ wParam, long /*int*/ lParam) {
 }
 
 @Override
-LRESULT WM_PALETTECHANGED (long /*int*/ wParam, long /*int*/ lParam) {
-	if (wParam != handle) {
-		long /*int*/ hPalette = display.hPalette;
-		if (hPalette != 0) return selectPalette (hPalette);
-	}
-	return super.WM_PALETTECHANGED (wParam, lParam);
-}
-
-@Override
-LRESULT WM_QUERYNEWPALETTE (long /*int*/ wParam, long /*int*/ lParam) {
-	long /*int*/ hPalette = display.hPalette;
-	if (hPalette != 0) return selectPalette (hPalette);
-	return super.WM_QUERYNEWPALETTE (wParam, lParam);
-}
-
-@Override
-LRESULT WM_SETCURSOR (long /*int*/ wParam, long /*int*/ lParam) {
+LRESULT WM_SETCURSOR (long wParam, long lParam) {
 	/*
 	* Feature in Windows.  When the shell is disabled
 	* by a Windows standard dialog (like a MessageBox
@@ -2421,14 +2382,14 @@ LRESULT WM_SETCURSOR (long /*int*/ wParam, long /*int*/ lParam) {
 		if (!Display.TrimEnabled) {
 			Shell modalShell = display.getModalShell ();
 			if (modalShell != null && !isActive ()) {
-				long /*int*/ hwndModal = modalShell.handle;
+				long hwndModal = modalShell.handle;
 				if (OS.IsWindowEnabled (hwndModal)) {
 					OS.SetActiveWindow (hwndModal);
 				}
 			}
 		}
 		if (!OS.IsWindowEnabled (handle)) {
-			long /*int*/ hwndPopup = OS.GetLastActivePopup (handle);
+			long hwndPopup = OS.GetLastActivePopup (handle);
 			if (hwndPopup != 0 && hwndPopup != handle) {
 				if (display.getControl (hwndPopup) == null) {
 					if (OS.IsWindowEnabled (hwndPopup)) {
@@ -2478,7 +2439,7 @@ LRESULT WM_SETCURSOR (long /*int*/ wParam, long /*int*/ lParam) {
 }
 
 @Override
-LRESULT WM_SHOWWINDOW (long /*int*/ wParam, long /*int*/ lParam) {
+LRESULT WM_SHOWWINDOW (long wParam, long lParam) {
 	LRESULT result = super.WM_SHOWWINDOW (wParam, lParam);
 	if (result != null) return result;
 	/*
@@ -2501,7 +2462,7 @@ LRESULT WM_SHOWWINDOW (long /*int*/ wParam, long /*int*/ lParam) {
 }
 
 @Override
-LRESULT WM_WINDOWPOSCHANGING (long /*int*/ wParam, long /*int*/ lParam) {
+LRESULT WM_WINDOWPOSCHANGING (long wParam, long lParam) {
 	LRESULT result = super.WM_WINDOWPOSCHANGING (wParam,lParam);
 	if (result != null) return result;
 	WINDOWPOS lpwp = new WINDOWPOS ();
